#!/opt/pmx6/bin/perl
# index.cgi
use HTML::Template;
use strict;
eval {use PureMessage::DelayQ};
use PureMessage::Manager::Config qw(%config);
use PureMessage::Message::ComputedFields;
use Time::Local;
use HTTP::Date;
use Data::Dump qw(dump);
use Carp qw(confess croak);
use POSIX qw(mktime ceil);
use PureMessage::Manager::Widget;
use Data::Dumper;				
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Response::DelayQ;
use PureMessage::Manager::Lang;
use PureMessage::Manager::Widget;

use PureMessage::Logger;
use ActiveState::UTF8 qw(maybe_decode_utf8 encode_utf8);
use PureMessage::MIME::Header qw(header_utf8);

use ActiveState::I18N::UTF8 qw(utf8_header utf8_length);
use PureMessage::UTF8::Config qw(%utf8_config);

my $logger = PureMessage::Logger->new('ident' => "delayq.cgi");

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::DelayQ',
                                'html_meta_charset' => 'utf-8',
                                'stderr_to_stdout' =>1
                                     );

$r->add_help(text => "Help", tab => "delay", page => "mgr_delay_over" );

my $WIDGET = new PureMessage::Manager::Widget;
$r->{debug} =0;


our %in;
our %text;
our $in;
our (@notifications);

# massage the in hash so that it is backwards compatible for the old parse
# routines. 
# flatten input from form buttons
foreach (qw(sm_retry sm_delete sm_delete_all sm_next_page sm_prev_page)) {
    $in{$_} = join("", @{$in{$_}}) if ref($in{$_}) eq "ARRAY";
}
foreach (keys %in) {
    delete($in{$_}) unless defined($in{$_}) and $in{$_} ne '';
}
my $sent_folder = 'Sent';

my $javascript = <<EOT;
<script>
var old_status;
var preview_window = null;

function changeAll(v) {
for(var i=0;i<document.query.elements.length;i++)
    if (document.query.elements[i].name.indexOf('c_row') != -1) {  
        document.query.elements[i].checked=v;
    }
}

function closePreview() {
    if (preview_window != null) {
        preview_window.close();
    }
}
</script>
EOT

my $in_body = <<EOT;
onUnload="closePreview()"
EOT

$r->add_head($javascript);
$r->body_attrib(onUnload=>'closePreview()');
$r->title(&text('index_title'));

open STDERR, ">&STDOUT";
select STDOUT; $|=1;

# action error ids
my $approve_err = "error_can_not_approve";
my $delete_err = "error_can_not_delete";
#
# template defaults
#
my $simple_params = {
    s_results_per_page => [ #10, 20, 50, 100, 500, 1000
                 {VALUE=>10, SEL=>'', LABEL=>'10 messages'},
                {VALUE=>20, SEL=>'selected', LABEL=>'20 messages'},
                {VALUE=>50, SEL=>'', LABEL=>'50 messages'},
                {VALUE=>100, SEL=>'', LABEL=>'100 messages'},
                {VALUE=>500, SEL=>'', LABEL=>'500 messages'},
                {VALUE=>1000, SEL=>'', LABEL=>'1000 messages'},
                ],	
			s_age => [ #5 min, 15 min, 30 min, 45 min, 60 min, all
                {VALUE=>5, SEL=>'', LABEL=>'5 min'},
                {VALUE=>15, SEL=>'', LABEL=>'15 min'},
                {VALUE=>30, SEL=>'', LABEL=>'30 min'},
                {VALUE=>45, SEL=>'', LABEL=>'45 min'},
                {VALUE=>60, SEL=>'selected', LABEL=>'60 min'},
                ], 				
   
};
my $show_advanced = (exists $in{advanced} and $in{advanced} =~ /^1/);

my $navgation_params = {};
my $page_nav_params = {
};
my $result_params = {};
my $query_cmd = { type =>'image', icon=>'search',text=>'Run Query',alt=>'Run Query',name=>'s_query'};
my $query_params = {};

my $query_mode = 0;

my %field_width = (
    FROM => 24,
    TO => 24,
    SUBJECT => 64,
);
my $show_results = 0;
my $show_query_form = 1;

# reload params upon reentry
if ($ENV{'REQUEST_METHOD'} eq 'POST') {
    $show_results = 1;
    # simple params                                  
    # select inputs
    for my $element (qw/s_age s_results_per_page/) {
        for my $node (@{$simple_params->{$element}}) {
            $node->{SEL} = '' if exists $in{$element};
            $node->{SEL} = 'selected' if exists $in{$element} and $in{$element} eq $node->{VALUE};
        }
    }
    # text inputs
    for my $element (qw/t_id t_recipient t_sender m_milter_host/) {
        $simple_params->{$element} = html_escape($in{$element}) if exists $in{$element};
    }
    
	if (exists $in{'s_query'}) {
        delete $in{$_} for (grep /^c_row/, keys %in);
        $in{this_page} = 0;
        $in{message_set_id} = 0;
    }
}
else {
    #print "<p><b>Welcome to the Delayed Mails Manager</b></p>";
    }
my $initialized = 0;
my $mstore;
my $create = (defined $in{create} and $in{create});

eval {
    $mstore = PureMessage::DelayQ->new(undef, create=>$create);
};

if ($@) {
    if ($@ =~ /^Malformed/) {
        push @notifications, <<EOT;
        <p>The Delayed Mails is currently uninitialized.
        This is probably because PureMessage was just installed.
        </p>
        <p>To initialize the Delayed Mails, click <a href="./index.cgi?create=1">initialize</a>.
        </p>

EOT
    }
    else {
        $r->raise_error ('error_unable_to_open_mstore',$@);
    }
}
else {
    $initialized = 1;
}

my $template;
sub print_query_form {
    my %data;
    #my @flds = qw(t_recipient t_sender t_id);
    my @flds = qw(t_recipient t_sender m_milter_host s_age);
    #### my @a_flds = qw(h_subject m_spam_hit m_reason m_relay m_milter_host s_age s_reason s_orderby s_groupby);
    my @extra;
    
    # the results per page field should always be last.
   # push(@flds, 's_results_per_page') unless $show_advanced;
    foreach (@flds) {
        if (ref($simple_params->{$_}) eq "ARRAY") {
            $data{$_} = combo($_);
        }
        else {
            $data{$_} = input($_);
        }
    }
     push(@flds, 's_results_per_page');
    $data{'s_results_per_page'} = combo('s_results_per_page');                
    my $tb = $r->ToolBar( buttons => [[ $query_cmd,]], );
    $r->print("<br><br><a name='queryform'>");
    $r->addInfoBox( msg => text('advanced_warning')) if $show_advanced;
    $r->addForm(
            data   => \%data,
            extra_rows => \@extra,
            name   => "query",
            fields => \@flds,
            state  => $show_advanced ? { "advanced" => 1 } : {},
            width  => "100%",
            columns => $show_advanced ? 3 : 2,
            title => text("query_form_subtitle"),
            toolbar => $tb,
            );

}

sub combo {
    my $name = shift;
    my $size = shift;
    my $class = "class=stretch" unless $size;
    my $html = "<select name='$name' $class>\n";
    my $list = $simple_params->{$name};
    foreach ( @{$list} ) {
        $html .= "<option $_->{SEL} value='$_->{VALUE}'>$_->{LABEL}</option>\n";
    }
    $html .= "</select>\n";
    return $html;
}

sub input {
    my $name = shift;
    my $size = shift;
    my $class = "class=stretch" unless $size;
    my $size = "size=$size" if $size;
    my $value = $simple_params->{$name};
    return "<input name='$name' $size value='$value' $class>";
}

#
# display output
#
if ($initialized) {
    $r->print('<form name="query" action="index.cgi" method="POST">');
}
        
# figure out what page we're on
my $results_per_page = $in{s_results_per_page} || 20;
my $rownum = $in{row_num} || 0;
my $last_rownum = $in{last_row_num} || 0;
my $this_page = $in{this_page} || 0;
my $message_set_id = $in{message_set_id} || undef;
$this_page-- if ($in{'sm_prev_page'} && $this_page);
$this_page++ if ($in{'sm_next_page'});
    
sub do_on_error {
    my $id = shift;    
    my $code = shift;
    my $info = shift;
    
    if ($code eq 'smtp_error') {
        $r->raise_error('error_action_smtp', $info);
    }
    else {
        $r->raise_error('error_action', $info);
    }
}

sub get_selected_rows {
    my @rows;
    for my $i (grep /c_row/, keys %in) {
        if ($in{$i} ne 'deleted') {
            $i =~ s/^c_row//;
            push @rows, $i;
        }
    }
    return @rows;
}

sub get_selected_message_ids {
    my $mset = shift;
    my @msgids;
    for my $i (get_selected_rows()) {
        my $row = $mset->fetch($i);
		###this code is written for sending location and  other info for  RPC call handling .at present we directly update dabatabase 
		#my @fields = $row->fields;
		#my $location=(defined $fields[5] and $fields[5])? $fields[5] : '';
		#my $delay_dir=(defined $fields[6] and $fields[6])? $fields[6] : '';
		#my $hostname=(defined $fields[7] and $fields[7])? $fields[7] : '';
		#my $hash={"msgid"=>$row->message_ids,"location"=>$location,"delayDir"=>$delay_dir,'hostname'=>$hostname};
        #push @msgids, $hash;
		push @msgids, $row->message_ids;        
    }
    return @msgids;
}

sub get_deleted_rows {
    my @rows;
    for my $i (grep /c_row/, keys %in) {
        if ($in{$i} eq 'deleted') {
            $i =~ s/^c_row//;
            push @rows, $i;
        }
    }
    return @rows;
}

my @deleted_rows;
sub save_deleted_rows {
    my $str = '';
    for my $i (@deleted_rows) {
        $str.="<input type=hidden name=c_row$i value=deleted>";
    }
    return $str;
}

sub get_all_message_ids {
    my $mset = shift;
    my @msgids;
    my $msgids = 0;
    my $i = 0;
    while (my $row = $mset->fetch($i++)) {
        next if grep $i == $_, @deleted_rows;
        if (wantarray) {
			###this code is written for sending location and  other info for  RPC call handling .at present we directly update dabatabase 
			#my @fields = $row->fields;
			#my $location=(defined $fields[5] and $fields[5])? $fields[5] : '';
			#my $delay_dir=(defined $fields[6] and $fields[6])? $fields[6] : '';
			#my $hostname=(defined $fields[7] and $fields[7])? $fields[7] : '';
			#my $hash={"msgid"=>$row->message_ids,"location"=>$location,"delayDir"=>$delay_dir,'hostname'=>$hostname};
			#push @msgids, $hash;
            push @msgids, $row->message_ids;
        }
        else {
            $msgids += scalar $row->message_ids;
        }
    }
    return @msgids if wantarray;
    return $msgids;
}

sub save_selected_rows {
    my $str;
    for my $i (grep /c_row/, keys %in) {
        if ($in{$i} ne 'deleted') {
            $str.="<input type=hidden name=$i value=".$in{$i}.">";
        }
    }
    return $str;
}

sub save_page_state {    
    return ""
    . "<input type=hidden name=this_page value=$this_page>"
    . "<input type=hidden name=message_set_id value=$message_set_id>"
    . "<input type=hidden name=row_num value=$rownum>"
    . "<input type=hidden name=last_row_num value=$last_rownum>"
    .save_deleted_rows();
}

sub get_admin {
    my $admin = $config{admin};
    if ($admin =~ /.*?<(.*)>/) {
        return $1;
    }
    return $admin;
}

sub get_mset {
    my $mstore = shift;
    # try to get a previous mset
    $r->raise_error('error_missing_parameter','message_set_id') unless $message_set_id;
    my $mset = $mstore->delayq_message_set(id=>$message_set_id);
    $r->raise_error('error_unable_to_open_mset',$message_set_id) unless $mset;
    return $mset;
}

# this is here to process a maximum number of messages at one time,
# since we currently have to open a file in order to process each message.
sub process_messages {
    my $mstore = shift;
    my $action = shift;    
    my $msgids = shift;
    my $error_id = shift;
    my @pass_thru = @_;
    my @to_process;
    my $msgs_processed = 0;
    my $max_msgs = 30;
    my $num_msgs = scalar @{$msgids};
    if ($num_msgs > $max_msgs) {
        my $step = $max_msgs;
        my $pos=0;
        while ($num_msgs) {
            $step = $num_msgs if $num_msgs < $step;
            #print " ($pos..",($pos+$step-1),")";
            my $last = $pos+$step-1;
            @to_process = @{$msgids}[$pos .. $last];
			my @msgs = @to_process;
            #my @msgs = $mstore->message(@to_process);
            last unless ($msgs_processed = $action->(\@msgs, logger => $logger, @pass_thru));
            $pos += $step;
            $num_msgs -= $step;
        }
    }
    else {
        ####my @msgs = $mstore->message(@{$msgids});
        my @msgs = @{$msgids};
        $msgs_processed = $action->(\@msgs, logger => $logger,@pass_thru);
    }
    $r->raise_error($error_id) unless $msgs_processed;
    return $msgs_processed;
}

my $do_action = 0;
my $mset;

@deleted_rows = get_deleted_rows();

# handle actions
use PureMessage::DMan qw(retry trash);

if ($in{'sm_retry'} || $in{'sm_delete'} || $in{'sm_cancel'} 
	|| $in{'sm_delete_all'}|| $in{'sm_do_delete_all'}){
    $do_action = 1;
    $mstore->message_writable(1);
    
    $mset = get_mset($mstore);
    my @msgs;
	if ($in{'sm_do_delete_all'}) {
		@msgs = get_all_message_ids($mset);
    }
	else {
		@msgs = get_selected_message_ids($mset);
    }
    ACTION: {
        ($in{'sm_retry'}) and do {
            $r->raise_error('error_no_messages_selected'), last ACTION unless @msgs;
            if (process_messages($mstore, \&retry, \@msgs, $approve_err,
                                     on_error=>\&do_on_error)) {
                    push @notifications, scalar @msgs. " message".((scalar @msgs)==1 ? '' : 's')." reinjected.";
                    push @deleted_rows, get_selected_rows();
		    $rownum = $last_rownum;
		    $last_rownum -= $results_per_page;
                }
            last ACTION;
        };
        
        ($in{'sm_delete_all'}) and do {
            $show_results=0;
            $show_query_form=0;
            $template = HTML::Template->new(filename => 'tmpl/qman-question.html');
            my $qparams = {
                SHOW_CONFIRM => 1,
                DESC =>   "This will delete all messages in your current query.  Are you sure?",
                SECTION_FOOTER => $r->ThemeSectionFooter( 
                    toolbar=> $r->ToolBar(
                      buttons => [[ { type=>'image',
                                      name=>'sm_do_delete_all',
                                      icon=>'delete',
                                      alt=>'Delete',
                                      text=>'Delete',
                                    },
                                    { type=>'image',
                                      name=>'sm_cancel',
                                      icon=>'cancel',
                                      alt=>'Cancel',
                                      text=>'Cancel',
                                    },
                                 ]],)
                    )->output(),
                SECTION_HEADER => $WIDGET->section_header("Confirm"),

            };
            $template->param($qparams);
            $r->add_part($template->output,'body_confirm_delete_all');
            $r->print(save_page_state());
            last ACTION;  
        };
        
        ($in{'sm_do_delete_all'}) and do {
            if (process_messages($mstore, \&trash, \@msgs, $delete_err)) {
                push @notifications, scalar @msgs. " message".((scalar @msgs)==1 ? '' : 's')." deleted.";
               # $mset->release;
               # undef($mset);
            }
            last ACTION;
        };
        
        ($in{'sm_cancel'}) and do {
            push @notifications, "Action cancelled.";
            last ACTION;
        };       
        ($in{'sm_delete'}) and do {
            $r->raise_error('error_no_messages_selected'), last ACTION unless @msgs;
            if (process_messages($mstore, \&trash, \@msgs, $delete_err)) {
                push @notifications, scalar @msgs. " message".((scalar @msgs)==1 ? '' : 's')." deleted.";
                push @deleted_rows, get_selected_rows();
				$rownum = $last_rownum;
				$last_rownum -= $results_per_page;
            }
            last ACTION;
        };           
       
    }
    # stay on the same page if necessary
    $rownum = $last_rownum if ($r->error() or $in{'sm_cancel'});
}

if ($create and $initialized) {
    push @notifications, "The Delayed Mail was initialized successfully";
}

$r->print("<p>" . $r->error_dlg_str() .  "<br></p>") if $r->error();

$r->print("<p>" . $r->notification_str($_) . "</p>")  while ($_ = shift @notifications);

# else Delayed Mail is empty
if (not defined $mstore) {
    # do nothing
}
elsif ($show_results) {
    
    if ($mset) {
        # do nothing, we've already got it
    }
    elsif ($in{'sm_prev_page'}
           || $in{'sm_next_page'}
           || $in{msg_preview_submit}
           ) {
        # try to get a previous mset
        $mset = get_mset($mstore);
    }
    # else do a new query
    else {
        #$this_page = 0;
	$rownum = 0;
	@deleted_rows = ();
	my $ct = time;
	
    my %opts;
	$opts{order_by} = ($in{s_orderby}) ? [$in{s_orderby}] : ['c_qdate_time desc'];
#        $opts{order_by} = ($in{s_orderby}) ? [$in{s_orderby}] : [];
         #### $opts{fields} = ['h_from','h_to','m_date','m_message_size','delayed_minute','location_name','delayed_dir','m_milter_host'];
        $opts{fields} = ['h_from','h_to','m_date','m_message_size','delayed_minute'];
        $opts{where} = '';
        my $and = '';

        # quote for MessageStore search syntax - so far just double up quotes.
        # MessageStore should handle the rest.
        # we need a quoting mechanism for % and _ ....
	#for my $field (qw/t_id m_relay m_spam_hit h_subject m_milter_host m_reason m_spam_probability_max m_spam_probability_min t_recipient t_sender/) {
        for my $field (qw/t_id m_relay m_milter_host m_reason t_recipient t_sender/) {
            $in{$field} =~ s/'/''/g if exists $in{$field};
	    $in{$field} =~ s/"/""/g if exists $in{$field};
            $in{$field} =~ s/\A\s*(.*?)\s*\z/$1/ if exists $in{$field}; # strip leading/trailing whitespace
        }

        # Quick fix to make subject querying work with latin-1 chars. 
        # When the display charset is changed to UTF-8, this should be 
        # taken out.
 	if (!$in{m_reason} and $in{s_reason} and ($in{s_reason} !~ /^any/)) {
		 $opts{where} .= $and."any m_reason like '$in{s_reason}'";
		 $and = ' and ';
	} 

        # match age less than ...
        if ($in{s_age} and not (defined $in{c_date_range} and $in{c_date_range} =~ /^on/)) {
            my $age = time - $in{s_age} * 60;
			$opts{where} .= $and.'c_qdate_time > '.$age.'';
	       $and = ' and ';
        }
        # match recipient against h_to, h_cc and m_envelope_to  (c_to index)
        if ($in{t_recipient}) {
            my $or = "";
            $opts{where} .= $and."(";
            $and = ' and ';                     
            for my $field (qw/c_to/) {
                if ($in{t_recipient} =~ /\%/) {
                    # e-mail address pattern match
                    $opts{where} .= $or." any $field like '" . lc($in{t_recipient}) . "'";
                }
                else {
                    # do exact match
                    $opts{where} .= $or." any $field = '" . lc($in{t_recipient}) . "'";
                }
                $or = " or ";
            }
            $opts{where} .= ")";
        }
        
         # match sender against h_from, m_envelope_from (c_from index)
        if ($in{t_sender}) {
            my $or = "";
            $opts{where} .= $and."(";
            $and = ' and ';                     
            for my $field (qw/c_from/) {
                if ($in{t_sender} =~ /\%/) {
                    # e-mail address pattern match
                    $opts{where} .= $or." any $field like '" . lc($in{t_sender}) ."'";
                }
                else {
                    # do exact match
                    $opts{where} .= $or." any $field = '" . lc($in{t_sender}) ."'";
                }
                $or = " or ";
            }
            $opts{where} .= ")";
        }       
        # match generic input fields
	#for my $field (qw/h_subject m_relay m_milter_host m_reason/) {
        for my $field (qw/m_relay m_milter_host/) {
            if ($in{$field}) {
                $opts{where} .= $and.qq{$field like "%$in{$field}%"};
                $and = ' and ';
            }
        }
        
	#print "WHERE: ", $opts{where}, "<br>\nFIELDS: ", dump($opts{fields}), "<br>\n";
	# set the result set limit to match the results per page
	#$opts{limit} = 0;
	$opts{limit} = $results_per_page;
        $opts{reverse} = 1;

        # if syntax and inputs are good..
        if(!$r->error()) {
                eval {
                 $mset = $mstore->delayq_message_set(%opts);
				#print "****** mset value:".Dumper($mset);
               };
                if ($@) {
                 $r->raise_error('error_creating_message_set',$@);
                } else {
                 # use process id as mset id for now
                 $message_set_id = $$;
                 $message_set_id = $mset->save($message_set_id);
                 $r->raise_error('unable_to_save_mset',$ENV{"SESSION_ID"}) unless defined $message_set_id;
                }
        }
        #print html_escape(dump(%opts));
    }

    #print "mset=$message_set_id";
    
    # DPL this block will have to be recoded when new mset is done
    my $tmpl_i = 0;
    #my @visible_rows;
    #my ($nrows,$numvisible) = (0,0);
    my $has_rows;
    
    if(!$r->error()) {
        #$nrows = scalar $mset->rows;
		$has_rows = $mset->rows;    
    } else {
        $r->error_dlg();
    }
    
    my %deleted_rows = map { $_ => 1 } @deleted_rows;
    # if there are some results
    #if ($numvisible > 0) {
    if ($has_rows) {
        
        #my $number_of_pages = ceil($numvisible  / $results_per_page);
        
        #if ($this_page == -1) {
        #    $this_page = $number_of_pages - 1;
        #}
        #else {
        #    $this_page-- while ($numvisible <= $results_per_page * $this_page);
        #}
        #my $start = $results_per_page*$this_page;
        #my $end = $numvisible < $results_per_page*($this_page+1) 
        #            ? $numvisible - 1 : $results_per_page*($this_page+1)-1;
        #            
        #my @rowset = @visible_rows[$start..$end];
        
        #print dump(@rowset);
        
	if ($in{sm_prev_page}) {
	    $rownum = $last_rownum - $results_per_page;
	}
	my $rowcount = $rownum;
	my $row;
	while ($row = $mset->fetch($rowcount++)) {
		if ($rowcount >= $results_per_page + $rownum + 1) {
		    last;
		}
		next if $deleted_rows{$rowcount - 1};
                my @fields = $row->fields;
                my $message_id = $row->message_id;
                my $messages_in_row = scalar $row->message_ids;
                ####$fields[0] .= sprintf ' %.1f%%',($fields[5]*100) if $fields[0] =~ /^spam$/i;
                
                # nice date 
                #my $date_str = str2time($fields[2]);
                my $date_str = ($fields[2]);
                if ($date_str) {
					my ($gyear, $gmonth, $gday, $ghour,$gmin, $gsec) = $date_str =~ /(\d+)-(\d\d)-(\d\d)\s+(\d\d):(\d\d):(\d\d)/;
					my $epoch = timegm ($gsec,$gmin,$ghour,$gday,$gmonth-1,$gyear);                    
					my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($epoch);
                    $min = '0'.$min if $min < 10;
                    $year+= 1900;
                    $wday = qw(Sun Mon Tue Wed Thu Fri Sat)[$wday];
                    $mon = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec)[$mon];
                    $date_str = "$wday, $mday $mon $hour:$min";
                }
				my $delayed_minute = ((defined $fields[4] and $fields[4]) ? $fields[4] : '');
               # my $minute_hour = str2time($fields[2]);  
				my $minute_hour = $fields[2];  				
				if ($minute_hour) {
					my ($gyear, $gmonth, $gday, $ghour,$gmin, $gsec) = $minute_hour =~ /(\d+)-(\d\d)-(\d\d)\s+(\d\d):(\d\d):(\d\d)/;
					my $epoch = timegm ($gsec,$gmin,$ghour,$gday,$gmonth-1,$gyear);
                    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($epoch);
                    $min=$min+$delayed_minute;
					if($min >=60){
						$min=$min%60;
						$hour=($hour+1)%24;
					}
					$min = '0'.$min if $min < 10;
					$minute_hour = "$hour:$min";
				}
                my %addr;
				$addr{from} = (defined $fields[0] and $fields[0])
					? $fields[0] : '';
				utf8_header($addr{from}, \%utf8_config);
				$addr{to} = (defined $fields[1] and $fields[1])
					? $fields[1] : '';
				utf8_header($addr{to}, \%utf8_config);
				my $date = $date_str || ((defined $fields[2] and $fields[2]) ? $fields[2] : '(none)');
				my $message_size = ((defined $fields[3] and $fields[3]) ? $fields[3] : '(none)');
				my $delayed_minute = ((defined $fields[4] and $fields[4]) ? $fields[4] : '');
               my %matches = ( from => $in{t_sender},
                                to => $in{t_recipient}
                              );
                foreach my $temp (keys %matches) {
                  $matches{$temp} =~ s/\%/.*/; 
                }
                
                foreach my $part (keys %addr) {
                        my @addys;
                        
                        @addys = split("\n",$addr{$part});
                        my @value;
                        foreach my $addr (@addys) {
                                if ($matches{$part} and $addr =~ /${matches{$part}}/ ) {
                                  unshift(@value,htmlify(scalar($addr)));
                                } else {
                                  push(@value,htmlify(scalar($addr)));
                                }
                        }
                        my $temp = (defined $value[0] and $value[0]) ? (substr $value[0],0, utf8_length($value[0], $field_width{uc($part)})) : '(none)';
			# XXX truncate the corrupted html-encoding character
			$temp =~ s/&[^;]*$//;
                        $addr{$part} ="<div title='" . non_breaking(join(' ',@value)) . "'>" . non_breaking($temp) . "</div>"; 
                }
                $result_params->{MESSAGES}->[$tmpl_i]->{FROM} = $addr{from};
                $result_params->{MESSAGES}->[$tmpl_i]->{TO} = $addr{to};
                $result_params->{MESSAGES}->[$tmpl_i]->{Size} = $message_size;
                $result_params->{MESSAGES}->[$tmpl_i]->{ROW}    = $rowcount - 1;
                $result_params->{MESSAGES}->[$tmpl_i]->{DATE} = non_breaking($date);
                $result_params->{MESSAGES}->[$tmpl_i]->{DELAYMIN} = $minute_hour;
                $tmpl_i++;
        }
        
  
        # save page state
	$last_rownum = $rownum;
	$rownum = $rowcount - 1;
	$message_set_id = $mset->save($message_set_id);
        $r->print(save_page_state());

        my $temp =  <<EOT;
        <div style="font-size:10pt"><a style="font-size:10pt" href='#queryform'>Edit this query</a> |
        <a style="font-size:10pt" href='./index.cgi'>New simple query</a> 
        </div>
        <br>
EOT
        $r->print($temp);
        # top page-nav bar
        my $page_nav_template = HTML::Template->new(filename => 'tmpl/qman-page-nav.html');
        
        $page_nav_params->{PREV} = button({type=>'text', name=>'sm_prev_page', text=>'Prev', alt=>'Previous'})
            if $rownum - $results_per_page > 0;
        $page_nav_params->{NEXT} = button({ type=>'text', name=>'sm_next_page', text=>'Next', alt=>'Next'})
          if $row;
        $page_nav_template->param($page_nav_params);

        $r->add_part($page_nav_template->output,'body_page_nav');
        $r->print("<br>");
        
        # top navigation bar
            
        if ($tmpl_i) {
            #print "<br>$first_row,$last_row,$i RESULT:".dump(keys %$_) foreach (@{$result_params->{MESSAGES}});
            # table of results
            $template = HTML::Template->new(filename => 'tmpl/qman-result.html', loop_context_vars=>1);
            my $tb = $r->ToolBar(buttons=>[[
                     { type=>'text', text=>'Reinject', name=>'sm_retry'},
                     { type=>'text', text=>'Delete', name=>'sm_delete'},
                     { type=>'text', text=>'Delete All', name=>'sm_delete_all'},
                                    ]]);

            $template->param({
                                HEADER => $r->ThemeSectionToolbar(align=>'top', toolbar=>$tb)->output(),
                                FOOTER => $r->ThemeSectionToolbar(toolbar=>$tb)->output(),
                             });
            $template->param($result_params);
            $r->add_part($template->output,'body_result');
        }
        else {
            push @notifications, "No messages left on this page";
        }
        
        # bottom navigation bar
        $page_nav_template->param($page_nav_params);
        $r->print("<br>");
        $r->add_part($page_nav_template->output,'body_page_nav');
                
    }
    elsif ($mstore->message_count == 0) {
        push @notifications, "The delayed Mail is empty";
    }    
    # else no results
    else {
        push @notifications, "No indexed messages matched this query. Most recently delayed Mail messages may have not been indexed yet.";
    }
    $r->print("<br><br>");
    print_query_form();
}
# first entry - just print form
else {
    print_query_form() if $show_query_form;
}
$r->print('</form>');

$r->error_dlg() if $r->error();

$r->print("<p>" . $r->notification_str($_) . "</p>")  while ($_ = shift @notifications);

$r->send();

#&footer("/", $text{'index'});


